@@ -2,13 +2,14 @@
 
 import json
 
-import pandas as pd
 from sqlglot import exp
 
 from sqlmesh.utils.migration import index_text_type
 
 
 def migrate(state_sync, **kwargs):  # type: ignore
+    import pandas as pd
+
     engine_adapter = state_sync.engine_adapter
     schema = state_sync.schema
     snapshots_table = "_snapshots"
@@ -43,7 +44,7 @@ def migrate(state_sync, **kwargs):  # type: ignore
         engine_adapter.insert_append(
             snapshots_table,
             pd.DataFrame(new_snapshots),
-            columns_to_types={
+            target_columns_to_types={
                 "name": exp.DataType.build(index_type),
                 "identifier": exp.DataType.build(index_type),
                 "version": exp.DataType.build(index_type),
