@@ -3,7 +3,6 @@ import typing as t
 from datetime import datetime
 from unittest.mock import call
 
-import pandas as pd
 import pytest
 from pyspark.sql import types as spark_types
 from pytest_mock.plugin import MockerFixture
@@ -11,12 +10,21 @@ from sqlglot import expressions as exp
 from sqlglot import parse_one
 
 from sqlmesh.core.engine_adapter import SparkEngineAdapter
+from sqlmesh.core.engine_adapter.shared import DataObject
 from sqlmesh.utils.errors import SQLMeshError
 from tests.core.engine_adapter import to_sql_calls
+import sqlmesh.core.dialect as d
+from sqlmesh.core.model import load_sql_based_model
+from sqlmesh.core.model.definition import SqlModel
 
 pytestmark = [pytest.mark.engine, pytest.mark.spark]
 
 
+@pytest.fixture
+def adapter(make_mocked_engine_adapter: t.Callable) -> SparkEngineAdapter:
+    return make_mocked_engine_adapter(SparkEngineAdapter)
+
+
 def test_create_table_properties(make_mocked_engine_adapter: t.Callable):
     adapter = make_mocked_engine_adapter(SparkEngineAdapter)
 
@@ -75,14 +83,14 @@ def test_replace_query_table_properties_not_exists(
     adapter.replace_query(
         "test_table",
         parse_one("SELECT 1 AS cola, '2' AS colb, '3' AS colc"),
-        columns_to_types=columns_to_types,
+        target_columns_to_types=columns_to_types,
         partitioned_by=[exp.to_column("colb")],
         storage_format="ICEBERG",
         table_properties={"a": exp.convert(1)},
     )
 
     assert to_sql_calls(adapter) == [
-        "CREATE TABLE IF NOT EXISTS `test_table` USING ICEBERG PARTITIONED BY (`colb`) TBLPROPERTIES ('a'=1) AS SELECT 1 AS `cola`, '2' AS `colb`, '3' AS `colc`",
+        "CREATE TABLE IF NOT EXISTS `test_table` USING ICEBERG PARTITIONED BY (`colb`) TBLPROPERTIES ('a'=1) AS SELECT CAST(`cola` AS INT) AS `cola`, CAST(`colb` AS STRING) AS `colb`, CAST(`colc` AS STRING) AS `colc` FROM (SELECT 1 AS `cola`, '2' AS `colb`, '3' AS `colc`) AS `_subquery`",
         "INSERT INTO `test_table` SELECT * FROM `test_table`",
     ]
 
@@ -95,6 +103,11 @@ def test_replace_query_table_properties_exists(
         return_value=True,
     )
     adapter = make_mocked_engine_adapter(SparkEngineAdapter)
+    mocker.patch.object(
+        adapter,
+        "_get_data_objects",
+        return_value=[DataObject(schema="", name="test_table", type="table")],
+    )
 
     columns_to_types = {
         "cola": exp.DataType.build("INT"),
@@ -104,7 +117,7 @@ def test_replace_query_table_properties_exists(
     adapter.replace_query(
         "test_table",
         parse_one("SELECT 1 AS cola, '2' AS colb, '3' AS colc"),
-        columns_to_types=columns_to_types,
+        target_columns_to_types=columns_to_types,
         partitioned_by=[exp.to_column("colb")],
         storage_format="ICEBERG",
         table_properties={"a": exp.convert(1)},
@@ -140,17 +153,16 @@ def test_alter_table(make_mocked_engine_adapter: t.Callable):
                 "complex": exp.DataType.build("STRUCT<complex_a: INT, complex_b: STRING>"),
                 "ds": exp.DataType.build("STRING"),
             }
-        else:
-            return {
-                "id": exp.DataType.build("BIGINT"),
-                "a": exp.DataType.build("STRING"),
-                "complex": exp.DataType.build("STRUCT<complex_a: INT>"),
-                "ds": exp.DataType.build("INT"),
-            }
+        return {
+            "id": exp.DataType.build("BIGINT"),
+            "a": exp.DataType.build("STRING"),
+            "complex": exp.DataType.build("STRUCT<complex_a: INT>"),
+            "ds": exp.DataType.build("INT"),
+        }
 
     adapter.columns = table_columns
 
-    adapter.alter_table(adapter.get_alter_expressions(current_table_name, target_table_name))
+    adapter.alter_table(adapter.get_alter_operations(current_table_name, target_table_name))
 
     adapter.cursor.execute.assert_has_calls(
         [
@@ -178,7 +190,7 @@ def test_replace_query_not_exists(mocker: MockerFixture, make_mocked_engine_adap
     )
 
     assert to_sql_calls(adapter) == [
-        "CREATE TABLE IF NOT EXISTS `test_table` AS SELECT `a` FROM `tbl`",
+        "CREATE TABLE IF NOT EXISTS `test_table` AS SELECT CAST(`a` AS INT) AS `a` FROM (SELECT `a` FROM `tbl`) AS `_subquery`",
     ]
 
 
@@ -188,6 +200,11 @@ def test_replace_query_exists(mocker: MockerFixture, make_mocked_engine_adapter:
         return_value=True,
     )
     adapter = make_mocked_engine_adapter(SparkEngineAdapter)
+    mocker.patch.object(
+        adapter,
+        "_get_data_objects",
+        return_value=[DataObject(schema="", name="test_table", type="table")],
+    )
     adapter.replace_query("test_table", parse_one("SELECT a FROM tbl"), {"a": "int"})
 
     assert to_sql_calls(adapter) == [
@@ -195,48 +212,6 @@ def test_replace_query_exists(mocker: MockerFixture, make_mocked_engine_adapter:
     ]
 
 
-def test_replace_query_pandas_not_exists(
-    make_mocked_engine_adapter: t.Callable, mocker: MockerFixture
-):
-    mocker.patch(
-        "sqlmesh.core.engine_adapter.spark.SparkEngineAdapter.table_exists",
-        return_value=False,
-    )
-    mocker.patch("sqlmesh.core.engine_adapter.spark.SparkEngineAdapter._use_spark_session", False)
-    mocker.patch(
-        "sqlmesh.core.engine_adapter.spark.SparkEngineAdapter._ensure_fqn", side_effect=lambda x: x
-    )
-    adapter = make_mocked_engine_adapter(SparkEngineAdapter)
-    df = pd.DataFrame({"a": [1, 2, 3], "b": [4, 5, 6]})
-    adapter.replace_query(
-        "test_table", df, {"a": exp.DataType.build("int"), "b": exp.DataType.build("int")}
-    )
-
-    assert to_sql_calls(adapter) == [
-        "CREATE TABLE IF NOT EXISTS `test_table` AS SELECT CAST(`a` AS INT) AS `a`, CAST(`b` AS INT) AS `b` FROM VALUES (1, 4), (2, 5), (3, 6) AS `t`(`a`, `b`)",
-    ]
-
-
-def test_replace_query_pandas_exists(make_mocked_engine_adapter: t.Callable, mocker: MockerFixture):
-    mocker.patch(
-        "sqlmesh.core.engine_adapter.spark.SparkEngineAdapter.table_exists",
-        return_value=True,
-    )
-    mocker.patch("sqlmesh.core.engine_adapter.spark.SparkEngineAdapter._use_spark_session", False)
-    mocker.patch(
-        "sqlmesh.core.engine_adapter.spark.SparkEngineAdapter._ensure_fqn", side_effect=lambda x: x
-    )
-    adapter = make_mocked_engine_adapter(SparkEngineAdapter)
-    df = pd.DataFrame({"a": [1, 2, 3], "b": [4, 5, 6]})
-    adapter.replace_query(
-        "test_table", df, {"a": exp.DataType.build("int"), "b": exp.DataType.build("int")}
-    )
-
-    assert to_sql_calls(adapter) == [
-        "INSERT OVERWRITE TABLE `test_table` (`a`, `b`) SELECT CAST(`a` AS INT) AS `a`, CAST(`b` AS INT) AS `b` FROM VALUES (1, 4), (2, 5), (3, 6) AS `t`(`a`, `b`)",
-    ]
-
-
 def test_replace_query_self_ref_not_exists(
     make_mocked_engine_adapter: t.Callable, mocker: MockerFixture, make_temp_table_name: t.Callable
 ):
@@ -275,12 +250,18 @@ def test_replace_query_self_ref_not_exists(
         side_effect=check_table_exists,
     )
 
+    mocker.patch.object(
+        adapter,
+        "_get_data_objects",
+        return_value=[DataObject(schema="db", name="table", type="table")],
+    )
+
     adapter.replace_query(table_name, parse_one(f"SELECT col + 1 AS col FROM {table_name}"))
 
     assert to_sql_calls(adapter) == [
         "CREATE TABLE IF NOT EXISTS `db`.`table` (`col` INT)",
         "CREATE SCHEMA IF NOT EXISTS `db`",
-        "CREATE TABLE IF NOT EXISTS `db`.`temp_table_abcdefgh` AS SELECT `col` FROM `db`.`table`",
+        "CREATE TABLE IF NOT EXISTS `db`.`temp_table_abcdefgh` AS SELECT CAST(`col` AS INT) AS `col` FROM (SELECT `col` FROM `db`.`table`) AS `_subquery`",
         "INSERT OVERWRITE TABLE `db`.`table` (`col`) SELECT `col` + 1 AS `col` FROM `db`.`temp_table_abcdefgh`",
         "DROP TABLE IF EXISTS `db`.`temp_table_abcdefgh`",
     ]
@@ -304,6 +285,11 @@ def test_replace_query_self_ref_exists(
 
     adapter = make_mocked_engine_adapter(SparkEngineAdapter)
     adapter.cursor.fetchone.return_value = (1,)
+    mocker.patch.object(
+        adapter,
+        "_get_data_objects",
+        return_value=[DataObject(schema="db", name="table", type="table")],
+    )
 
     table_name = "db.table"
     temp_table_id = "abcdefgh"
@@ -324,7 +310,7 @@ def test_replace_query_self_ref_exists(
         # This is something temp_table does and isn't really needed. The contract with `replace_query` is that
         # the schema for the table already exists
         "CREATE SCHEMA IF NOT EXISTS `db`",
-        f"CREATE TABLE IF NOT EXISTS `db`.`temp_table_{temp_table_id}` AS SELECT `col` FROM `db`.`table`",
+        f"CREATE TABLE IF NOT EXISTS `db`.`temp_table_{temp_table_id}` AS SELECT CAST(`col` AS INT) AS `col` FROM (SELECT `col` FROM `db`.`table`) AS `_subquery`",
         "INSERT OVERWRITE TABLE `db`.`table` (`col`) SELECT `col` + 1 AS `col` FROM `db`.`temp_table_abcdefgh`",
         f"DROP TABLE IF EXISTS `db`.`temp_table_{temp_table_id}`",
     ]
@@ -369,7 +355,6 @@ test_primitive_params = [
     ("double", spark_types.DoubleType()),
     ("decimal", spark_types.DecimalType()),
     ("text", spark_types.StringType()),
-    # Spark supports VARCHAR and CHAR but SQLGlot currently converts them to strings
     ("varchar(25)", spark_types.StringType()),
     ("char(30)", spark_types.StringType()),
     ("binary", spark_types.BinaryType()),
@@ -536,7 +521,13 @@ def test_spark_struct_primitives_to_col_to_types(type_name, spark_type):
     actual = SparkEngineAdapter.spark_to_sqlglot_types(
         spark_types.StructType([spark_types.StructField(f"col_{type_name}", spark_type)])
     )
-    expected = {f"col_{type_name}": exp.DataType.build(type_name, dialect="spark")}
+
+    expected_type = (
+        exp.DataType.build("string")
+        if "char" in type_name
+        else exp.DataType.build(type_name, dialect="spark")
+    )
+    expected = {f"col_{type_name}": expected_type}
     assert actual == expected
 
 
@@ -556,12 +547,8 @@ def test_spark_struct_complex_to_col_to_types(type_name, spark_type):
 def test_scd_type_2_by_time(
     make_mocked_engine_adapter: t.Callable, make_temp_table_name: t.Callable, mocker: MockerFixture
 ):
-    mocker.patch(
-        "sqlmesh.core.engine_adapter.spark.SparkEngineAdapter.table_exists",
-        return_value=False,
-    )
-
     adapter = make_mocked_engine_adapter(SparkEngineAdapter)
+    adapter._default_catalog = "spark_catalog"
     adapter.spark.catalog.currentCatalog.return_value = "spark_catalog"
     adapter.spark.catalog.currentDatabase.return_value = "default"
 
@@ -580,6 +567,11 @@ def test_scd_type_2_by_time(
         "sqlmesh.core.engine_adapter.spark.SparkEngineAdapter.table_exists",
         side_effect=check_table_exists,
     )
+    mocker.patch.object(
+        adapter,
+        "_get_data_objects",
+        return_value=[DataObject(schema="db", name="target", type="table")],
+    )
 
     adapter.scd_type_2_by_time(
         target_table="db.target",
@@ -590,7 +582,7 @@ def test_scd_type_2_by_time(
         valid_from_col=exp.column("test_valid_from", quoted=True),
         valid_to_col=exp.column("test_valid_to", quoted=True),
         updated_at_col=exp.column("test_updated_at", quoted=True),
-        columns_to_types={
+        target_columns_to_types={
             "id": exp.DataType.build("INT"),
             "name": exp.DataType.build("VARCHAR"),
             "price": exp.DataType.build("DOUBLE"),
@@ -604,21 +596,21 @@ def test_scd_type_2_by_time(
     assert to_sql_calls(adapter) == [
         "CREATE TABLE IF NOT EXISTS `db`.`target` (`id` INT, `name` STRING, `price` DOUBLE, `test_updated_at` TIMESTAMP, `test_valid_from` TIMESTAMP, `test_valid_to` TIMESTAMP)",
         "CREATE SCHEMA IF NOT EXISTS `db`",
-        "CREATE TABLE IF NOT EXISTS `db`.`temp_target_abcdefgh` AS SELECT `id`, `name`, `price`, `test_updated_at`, `test_valid_from`, `test_valid_to` FROM `db`.`target`",
+        "CREATE TABLE IF NOT EXISTS `db`.`temp_target_abcdefgh` AS SELECT CAST(`id` AS INT) AS `id`, CAST(`name` AS STRING) AS `name`, CAST(`price` AS DOUBLE) AS `price`, CAST(`test_updated_at` AS TIMESTAMP) AS `test_updated_at`, CAST(`test_valid_from` AS TIMESTAMP) AS `test_valid_from`, CAST(`test_valid_to` AS TIMESTAMP) AS `test_valid_to` FROM (SELECT `id`, `name`, `price`, `test_updated_at`, `test_valid_from`, `test_valid_to` FROM `db`.`target`) AS `_subquery`",
         parse_one(
             """WITH `source` AS (
   SELECT
-    TRUE AS `_exists`,
+    `_exists`,
     `id`,
     `name`,
     `price`,
-    CAST(`test_updated_at` AS TIMESTAMP) AS `test_updated_at`
+    `test_updated_at`
   FROM (
     SELECT
       TRUE AS `_exists`,
-      `id`,
-      `name`,
-      `price`,
+      `id` AS `id`,
+      `name` AS `name`,
+      `price` AS `price`,
       CAST(`test_updated_at` AS TIMESTAMP) AS `test_updated_at`,
       ROW_NUMBER() OVER (PARTITION BY COALESCE(`id`, '') ORDER BY COALESCE(`id`, '')) AS _row_number
     FROM (
@@ -639,7 +631,8 @@ def test_scd_type_2_by_time(
     `price`,
     `test_updated_at`,
     `test_valid_from`,
-    `test_valid_to`
+    `test_valid_to`,
+    TRUE AS `_exists`
   FROM `db`.`temp_target_abcdefgh`
   WHERE
     NOT `test_valid_to` IS NULL
@@ -650,7 +643,8 @@ def test_scd_type_2_by_time(
     `price`,
     `test_updated_at`,
     `test_valid_from`,
-    `test_valid_to`
+    `test_valid_to`,
+    TRUE AS `_exists`
   FROM `db`.`temp_target_abcdefgh`
   WHERE
     `test_valid_to` IS NULL
@@ -677,7 +671,7 @@ def test_scd_type_2_by_time(
     COALESCE(`id`, '')
 ), `joined` AS (
   SELECT
-    `source`.`_exists`,
+    `source`.`_exists` AS `_exists`,
     `latest`.`id` AS `t_id`,
     `latest`.`name` AS `t_name`,
     `latest`.`price` AS `t_price`,
@@ -691,9 +685,9 @@ def test_scd_type_2_by_time(
   FROM `latest`
   LEFT JOIN `source`
     ON COALESCE(`latest`.`id`, '') = COALESCE(`source`.`id`, '')
-  UNION
+  UNION ALL
   SELECT
-    `source`.`_exists`,
+    `source`.`_exists` AS `_exists`,
     `latest`.`id` AS `t_id`,
     `latest`.`name` AS `t_name`,
     `latest`.`price` AS `t_price`,
@@ -707,6 +701,8 @@ def test_scd_type_2_by_time(
   FROM `latest`
   RIGHT JOIN `source`
     ON COALESCE(`latest`.`id`, '') = COALESCE(`source`.`id`, '')
+  WHERE
+    `latest`.`_exists` IS NULL
 ), `updated_rows` AS (
   SELECT
     COALESCE(`joined`.`t_id`, `joined`.`id`) AS `id`,
@@ -725,8 +721,8 @@ def test_scd_type_2_by_time(
       ELSE `t_test_valid_from`
     END AS `test_valid_from`,
     CASE
-      WHEN `test_updated_at` > `t_test_updated_at`
-      THEN `test_updated_at`
+      WHEN `joined`.`test_updated_at` > `joined`.`t_test_updated_at`
+      THEN `joined`.`test_updated_at`
       WHEN `joined`.`_exists` IS NULL
       THEN CAST('2020-01-01 00:00:00' AS TIMESTAMP)
       ELSE `t_test_valid_to`
@@ -744,7 +740,7 @@ def test_scd_type_2_by_time(
     CAST(NULL AS TIMESTAMP) AS `test_valid_to`
   FROM `joined`
   WHERE
-    `test_updated_at` > `t_test_updated_at`
+    `joined`.`test_updated_at` > `joined`.`t_test_updated_at`
 )
 INSERT OVERWRITE TABLE `db`.`target` (
   `id`,
@@ -906,7 +902,7 @@ def test_comments_hive(mocker: MockerFixture, make_mocked_engine_adapter: t.Call
     sql_calls = to_sql_calls(adapter)
     assert sql_calls == [
         f"CREATE TABLE IF NOT EXISTS `test_table` (`a` INT COMMENT '{truncated_column_comment}', `b` INT) COMMENT '{truncated_table_comment}'",
-        f"CREATE TABLE IF NOT EXISTS `test_table` COMMENT '{truncated_table_comment}' AS SELECT `a`, `b` FROM `source_table`",
+        f"CREATE TABLE IF NOT EXISTS `test_table` COMMENT '{truncated_table_comment}' AS SELECT CAST(`a` AS INT) AS `a`, CAST(`b` AS INT) AS `b` FROM (SELECT `a`, `b` FROM `source_table`) AS `_subquery`",
         f"ALTER TABLE `test_table` ALTER COLUMN `a` COMMENT '{truncated_column_comment}'",
         f"CREATE OR REPLACE VIEW test_view COMMENT '{truncated_table_comment}' AS SELECT a, b FROM source_table",
         f"COMMENT ON TABLE `test_table` IS '{truncated_table_comment}'",
@@ -964,7 +960,7 @@ def test_comments_iceberg(mocker: MockerFixture, make_mocked_engine_adapter: t.C
     sql_calls = to_sql_calls(adapter)
     assert sql_calls == [
         f"CREATE TABLE IF NOT EXISTS `test_table` (`a` INT COMMENT '{long_column_comment}', `b` INT) COMMENT '{long_table_comment}'",
-        f"CREATE TABLE IF NOT EXISTS `test_table` COMMENT '{long_table_comment}' AS SELECT `a`, `b` FROM `source_table`",
+        f"CREATE TABLE IF NOT EXISTS `test_table` COMMENT '{long_table_comment}' AS SELECT CAST(`a` AS INT) AS `a`, CAST(`b` AS INT) AS `b` FROM (SELECT `a`, `b` FROM `source_table`) AS `_subquery`",
         f"ALTER TABLE `test_table` ALTER COLUMN `a` COMMENT '{long_column_comment}'",
         f"CREATE OR REPLACE VIEW test_view COMMENT '{long_table_comment}' AS SELECT a, b FROM source_table",
         f"COMMENT ON TABLE `test_table` IS '{long_table_comment}'",
@@ -1005,19 +1001,82 @@ def test_replace_query_with_wap_self_reference(
     )
 
     adapter = make_mocked_engine_adapter(SparkEngineAdapter)
+    mocker.patch.object(
+        adapter,
+        "_get_data_objects",
+        return_value=[DataObject(schema="schema", name="table", type="table")],
+    )
 
     adapter.replace_query(
         "catalog.schema.table.branch_wap_12345",
         parse_one("SELECT 1 as a FROM catalog.schema.table.branch_wap_12345"),
-        columns_to_types={"a": exp.DataType.build("INT")},
+        target_columns_to_types={"a": exp.DataType.build("INT")},
         storage_format="ICEBERG",
     )
 
     sql_calls = to_sql_calls(adapter)
     assert sql_calls == [
-        "CREATE TABLE IF NOT EXISTS `catalog`.`schema`.`table` (`a` INT)",
+        "CREATE TABLE IF NOT EXISTS `catalog`.`schema`.`table` (`a` INT) USING ICEBERG",
         "CREATE SCHEMA IF NOT EXISTS `catalog`.`schema`",
-        "CREATE TABLE IF NOT EXISTS `catalog`.`schema`.`temp_branch_wap_12345_abcdefgh` USING ICEBERG AS SELECT `a` FROM `catalog`.`schema`.`table`.`branch_wap_12345`",
+        "CREATE TABLE IF NOT EXISTS `catalog`.`schema`.`temp_branch_wap_12345_abcdefgh` USING ICEBERG AS SELECT CAST(`a` AS INT) AS `a` FROM (SELECT `a` FROM `catalog`.`schema`.`table`.`branch_wap_12345`) AS `_subquery`",
         "INSERT OVERWRITE TABLE `catalog`.`schema`.`table`.`branch_wap_12345` (`a`) SELECT 1 AS `a` FROM `catalog`.`schema`.`temp_branch_wap_12345_abcdefgh`",
         "DROP TABLE IF EXISTS `catalog`.`schema`.`temp_branch_wap_12345_abcdefgh`",
     ]
+
+
+def test_table_format(adapter: SparkEngineAdapter, mocker: MockerFixture):
+    mocker.patch(
+        "sqlmesh.core.engine_adapter.spark.SparkEngineAdapter.table_exists",
+        return_value=True,
+    )
+
+    expressions = d.parse(
+        """
+        MODEL (
+            name test_table,
+            kind FULL,
+            table_format iceberg,
+            storage_format orc
+        );
+
+        SELECT 1::timestamp AS cola, 2::varchar as colb, 'foo' as colc;
+    """
+    )
+    model: SqlModel = t.cast(SqlModel, load_sql_based_model(expressions))
+
+    # both table_format and storage_format
+    adapter.create_table(
+        table_name=model.name,
+        target_columns_to_types=model.columns_to_types_or_raise,
+        table_format=model.table_format,
+        storage_format=model.storage_format,
+    )
+
+    # just table_format
+    adapter.create_table(
+        table_name=model.name,
+        target_columns_to_types=model.columns_to_types_or_raise,
+        table_format=model.table_format,
+    )
+
+    # just storage_format set to a table format (test for backwards compatibility)
+    adapter.create_table(
+        table_name=model.name,
+        target_columns_to_types=model.columns_to_types_or_raise,
+        storage_format=model.table_format,
+    )
+
+    adapter.ctas(
+        table_name=model.name,
+        query_or_df=model.query,
+        target_columns_to_types=model.columns_to_types_or_raise,
+        table_format=model.table_format,
+        storage_format=model.storage_format,
+    )
+
+    assert to_sql_calls(adapter) == [
+        "CREATE TABLE IF NOT EXISTS `test_table` (`cola` TIMESTAMP, `colb` STRING, `colc` STRING) USING ICEBERG TBLPROPERTIES ('write.format.default'='orc')",
+        "CREATE TABLE IF NOT EXISTS `test_table` (`cola` TIMESTAMP, `colb` STRING, `colc` STRING) USING ICEBERG",
+        "CREATE TABLE IF NOT EXISTS `test_table` (`cola` TIMESTAMP, `colb` STRING, `colc` STRING) USING ICEBERG",
+        "CREATE TABLE IF NOT EXISTS `test_table` USING ICEBERG TBLPROPERTIES ('write.format.default'='orc') AS SELECT CAST(`cola` AS TIMESTAMP) AS `cola`, CAST(`colb` AS STRING) AS `colb`, CAST(`colc` AS STRING) AS `colc` FROM (SELECT CAST(1 AS TIMESTAMP) AS `cola`, CAST(2 AS STRING) AS `colb`, 'foo' AS `colc`) AS `_subquery`",
+    ]
